@@ -25,11 +25,10 @@
  * nm-connection-editor, and gnome-control-center.
  */
 
-#include "config.h"
-
-#include <NetworkManager.h>
-
 #include "nm-default.h"
+
+#include "NetworkManager.h"
+
 #include "nm-editor-utils.h"
 #if 0
 #include "nm-vpn-helpers.h"
@@ -224,7 +223,7 @@ nm_editor_utils_get_connection_type_list (void)
 
 #if 0
 	/* Add "VPN" only if there are plugins */
-	vpn_plugins_hash = nm_vpn_get_plugins ();
+	vpn_plugins_hash = nm_vpn_get_plugin_infos ();
 	have_vpn_plugins  = vpn_plugins_hash && g_hash_table_size (vpn_plugins_hash);
 	if (have_vpn_plugins) {
 		GHashTableIter iter;
@@ -253,6 +252,18 @@ nm_editor_utils_get_connection_type_list (void)
 	return list;
 }
 
+static gboolean
+_assert_format_int (const char *format)
+{
+	g_assert (format);
+	format = strchr (format, '%');
+	g_assert (format);
+	format++;
+	g_assert (!strchr (format, '%'));
+	g_assert (format[0] == 'd');
+	return TRUE;
+}
+
 static char *
 get_available_connection_name (const char *format,
                                NMClient   *client)
@@ -262,6 +273,8 @@ get_available_connection_name (const char *format,
 	char *cname = NULL;
 	int i = 0;
 
+	nm_assert (_assert_format_int (format));
+
 	conns = nm_client_get_connections (client);
 	for (i = 0; i < conns->len; i++) {
 		const char *id;
@@ -276,7 +289,9 @@ get_available_connection_name (const char *format,
 		char *temp;
 		gboolean found = FALSE;
 
+		NM_PRAGMA_WARNING_DISABLE("-Wformat-nonliteral")
 		temp = g_strdup_printf (format, i);
+		NM_PRAGMA_WARNING_REENABLE
 		for (iter = names; iter; iter = g_slist_next (iter)) {
 			if (!strcmp (iter->data, temp)) {
 				found = TRUE;
