@@ -89,6 +89,8 @@ enum {
 	PERMISSION_CHANGED,
 	CONNECTION_ADDED,
 	CONNECTION_REMOVED,
+	ACTIVE_CONNECTION_ADDED,
+	ACTIVE_CONNECTION_REMOVED,
 
 	LAST_SIGNAL
 };
@@ -1189,6 +1191,9 @@ nm_client_deactivate_connection_finish (NMClient *client,
  * Returns: (transfer none) (element-type NMRemoteConnection): an array
  * containing all connections provided by the remote settings service.  The
  * returned array is owned by the #NMClient object and should not be modified.
+ *
+ * The connections are as received from D-Bus and might not validate according
+ * to nm_connection_verify().
  **/
 const GPtrArray *
 nm_client_get_connections (NMClient *client)
@@ -1207,6 +1212,9 @@ nm_client_get_connections (NMClient *client)
  *
  * Returns: (transfer none): the remote connection object on success, or %NULL if no
  *  matching object was found.
+ *
+ * The connection is as received from D-Bus and might not validate according
+ * to nm_connection_verify().
  **/
 NMRemoteConnection *
 nm_client_get_connection_by_id (NMClient *client, const char *id)
@@ -1226,6 +1234,9 @@ nm_client_get_connection_by_id (NMClient *client, const char *id)
  *
  * Returns: (transfer none): the remote connection object on success, or %NULL if the object was
  *  not known
+ *
+ * The connection is as received from D-Bus and might not validate according
+ * to nm_connection_verify().
  **/
 NMRemoteConnection *
 nm_client_get_connection_by_path (NMClient *client, const char *path)
@@ -1245,6 +1256,9 @@ nm_client_get_connection_by_path (NMClient *client, const char *path)
  *
  * Returns: (transfer none): the remote connection object on success, or %NULL if the object was
  *  not known
+ *
+ * The connection is as received from D-Bus and might not validate according
+ * to nm_connection_verify().
  **/
 NMRemoteConnection *
 nm_client_get_connection_by_uuid (NMClient *client, const char *uuid)
@@ -1739,6 +1753,22 @@ settings_connection_removed (NMRemoteSettings *manager,
 	g_signal_emit (client, signals[CONNECTION_REMOVED], 0, connection);
 }
 
+static void
+manager_active_connection_added (NMManager *manager,
+                                 NMActiveConnection *active_connection,
+                                 gpointer client)
+{
+	g_signal_emit (client, signals[ACTIVE_CONNECTION_ADDED], 0, active_connection);
+}
+
+static void
+manager_active_connection_removed (NMManager *manager,
+                                   NMActiveConnection *active_connection,
+                                   gpointer client)
+{
+	g_signal_emit (client, signals[ACTIVE_CONNECTION_REMOVED], 0, active_connection);
+}
+
 static void
 constructed (GObject *object)
 {
@@ -1760,6 +1790,10 @@ constructed (GObject *object)
 	                  G_CALLBACK (manager_any_device_removed), client);
 	g_signal_connect (priv->manager, "permission-changed",
 	                  G_CALLBACK (manager_permission_changed), client);
+	g_signal_connect (priv->manager, "active-connection-added",
+	                  G_CALLBACK (manager_active_connection_added), client);
+	g_signal_connect (priv->manager, "active-connection-removed",
+	                  G_CALLBACK (manager_active_connection_removed), client);
 
 	priv->settings = g_object_new (NM_TYPE_REMOTE_SETTINGS,
 	                               NM_OBJECT_PATH, NM_DBUS_PATH_SETTINGS,
@@ -2341,6 +2375,36 @@ nm_client_class_init (NMClientClass *client_class)
 		              NULL, NULL, NULL,
 		              G_TYPE_NONE, 1,
 		              NM_TYPE_REMOTE_CONNECTION);
+
+	/**
+	 * NMClient::active-connection-added:
+	 * @client: the settings object that received the signal
+	 * @active_connection: the new active connection
+	 *
+	 * Notifies that a #NMActiveConnection has been added.
+	 **/
+	signals[ACTIVE_CONNECTION_ADDED] =
+		g_signal_new (NM_CLIENT_ACTIVE_CONNECTION_ADDED,
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_FIRST,
+		              0, NULL, NULL, NULL,
+		              G_TYPE_NONE, 1,
+		              NM_TYPE_ACTIVE_CONNECTION);
+
+	/**
+	 * NMClient::active-connection-removed:
+	 * @client: the settings object that received the signal
+	 * @active_connection: the removed active connection
+	 *
+	 * Notifies that a #NMActiveConnection has been removed.
+	 **/
+	signals[ACTIVE_CONNECTION_REMOVED] =
+		g_signal_new (NM_CLIENT_ACTIVE_CONNECTION_REMOVED,
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_FIRST,
+		              0, NULL, NULL, NULL,
+		              G_TYPE_NONE, 1,
+		              NM_TYPE_ACTIVE_CONNECTION);
 }
 
 static void
