@@ -55,35 +55,26 @@
 /* Not exported */
 #define NM_MANAGER_HOSTNAME "hostname"
 #define NM_MANAGER_SLEEPING "sleeping"
-#define NM_MANAGER_STATE_FILE "state-file"
+
+/* signals */
+#define NM_MANAGER_CHECK_PERMISSIONS         "check-permissions"
+#define NM_MANAGER_DEVICE_ADDED              "device-added"
+#define NM_MANAGER_DEVICE_REMOVED            "device-removed"
+#define NM_MANAGER_STATE_CHANGED             "state-changed"
+#define NM_MANAGER_USER_PERMISSIONS_CHANGED  "user-permissions-changed"
 
 /* Internal signals */
 #define NM_MANAGER_ACTIVE_CONNECTION_ADDED   "active-connection-added"
 #define NM_MANAGER_ACTIVE_CONNECTION_REMOVED "active-connection-removed"
 #define NM_MANAGER_CONFIGURE_QUIT            "configure-quit"
-#define NM_MANAGER_STATE_CHANGED             "state-changed"
+#define NM_MANAGER_INTERNAL_DEVICE_ADDED     "internal-device-added"
+#define NM_MANAGER_INTERNAL_DEVICE_REMOVED   "internal-device-removed"
 
 
-struct _NMManager {
-	NMExportedObject parent;
-};
-
-typedef struct {
-	NMExportedObjectClass parent;
-
-	/* Signals */
-	void (*device_added) (NMManager *manager, NMDevice *device);
-	void (*device_removed) (NMManager *manager, NMDevice *device);
-	void (*state_changed) (NMManager *manager, guint state);
-} NMManagerClass;
-
 GType nm_manager_get_type (void);
 
 /* nm_manager_setup() should only be used by main.c */
-NMManager *   nm_manager_setup                         (const char *state_file,
-                                                        gboolean initial_net_enabled,
-                                                        gboolean initial_wifi_enabled,
-                                                        gboolean initial_wwan_enabled);
+NMManager *   nm_manager_setup                         (void);
 
 NMManager *   nm_manager_get                           (void);
 
